package HyperEps.publish;

import java.rmi.UnexpectedException;
import java.util.List;

class Set {

  // private properties
  Point point; // the reference point from the set B
  int begin; // the begin index from the set A
  int end;  // the end index from the set A
  boolean increasing; // whether the indicator function values are increasing or decreasing for increasing x
  Point[] A;

  Set (Point reference, Point[] A, int a, int b, boolean increasing){
    this.point = reference;
    this.A = A;
    this.begin = a;
    this.end = b;
    this.increasing = !increasing;
  }

  /**
   * Function for debugging, returns all elements in a given set
   */
  void print() {
    String str = "";
    System.out.println("Increasing : " + increasing);
    for (int n=0; n <getSize(); n++ ) {
      str+=" " + getElement(n);
    }

    System.out.println(str);
  }

  /**
   * @return the number of elements in this set
   */
  public int getSize() {
    if (begin == -1 || end < begin) {
      return 0;
    } else {
      return end - begin + 1;
    }
  }

  /**
   * returns the element at the index [pos]
   */
  double getElement(int pos) {
    if (begin + pos > end) {
      System.out.println("This should not happen");
    }
    return EPSSSP.I(A[begin + pos], point);
  }

  /**
   * Deletes all elements bigger than d in the set
   * runtime O(n log n) where n is the size of the set
   */
  void deleteBigger(double d) {
    if (increasing) {

      if(end>=begin && EPSSSP.I(A[end], point) >= d) {
        int a = begin - 1;
        int b = end;
        while(a + 1 < b) {
          int mid = (a + b) / 2;

          if (EPSSSP.I(A[mid], point) >= d) {
            b = mid;
          } else {
            a = mid;
          }
        }

        end = b - 1;
      }

    } else {

      if(begin<=end && EPSSSP.I(A[begin], point) >= d) {
        int a = begin;
        int b = end + 1;

        while (a < b) {
          int mid = (a + b) / 2;

          if (EPSSSP.I(A[mid], point) >= d) {
            a = mid + 1;
          } else {
            b = mid;
          }
        }

        begin = b;
      }
    }
  }

  /**
   * Deletes all elements smaller than d in the set
   */
  void deleteSmaller(double d) {
    if (!increasing) {

      if(end>=begin && EPSSSP.I(A[end], point) <= d) {
        int a = begin - 1;
        int b = end;
        while(a + 1 < b) {
          int mid = (a + b) / 2;

          if (EPSSSP.I(A[mid], point) <= d) {
            b = mid;
          } else {
            a = mid;
          }
        }

        end = b - 1;
      }

    } else {

      if(begin<=end && EPSSSP.I(A[begin], point) <= d) {
        int a = begin;
        int b = end + 1;

        while (a < b) {
          int mid = (a + b) / 2;

          if (EPSSSP.I(A[mid], point) <= d) {
            a = mid + 1;
          } else {
            b = mid;
          }
        }

        begin = b;
      }
    }
  }

};
